//
// Implementation of TwitterFox content JS
//
// Copyright (c) 2009 Kazuho Okui / naan studio, Inc. ALL RIGHTS RESERVED.
//

var TWITTER_TOP_URL = "http://twitter.com/";
var TINYURL_CREATE_API = "http://tinyurl.com/api-create.php?url=";
var TWITTERFOX_URL_REGEXP = /((https?\:\/\/|www\.)[^\s]+)([^\w\s\d]*)/g;

function TwitterNotifier() {
  this._prefWindow = null;
  this._messageQueue = new Array();
  this._util = new naanExUtils("twitternotifier");
  this._hasFocus = false;
  this._focusInText = false;
  this._inReplyTo = 0;
}

TwitterNotifier.prototype = {

  load: function() {

    this._hasFocus = true;

    this.initKeyConfig();

    try {
      var fontSize = this._util.pref().getIntPref("fontSize");
      if (fontSize == 0) {
        if (navigator.platform.match("Mac")) {
          this._util.pref().setIntPref("fontSize", 11);
        }
        else {
          this._util.pref().setIntPref("fontSize", 12);
        }
      }
    }
    catch (e) {
        this._util.pref().setIntPref("fontSize", 11);
    }
    
    if (navigator.platform.match("Linux")) {
      this._util.pref().setBoolPref("dontPopupWhileTyping", true);
    }
    // Don't init TwitterFox when the window is popup.
    if (window.toolbar.visible == false) {
      var btn = this.$("twitternotifier-statusbar-button");;
      var parent = btn.parentNode;
      parent.removeChild(btn);
      return;
    }

    var observer = this.Cc["@mozilla.org/observer-service;1"].getService(this.Ci.nsIObserverService);
    observer.addObserver(gTwitterNotifier, "twitternotifier-status", false);

    // Setup menuitem
    var menu = this.$("twitternotifier-menuitem-togglepopup");
    menu.setAttribute("checked", this._util.pref().getBoolPref("popup"));

    this._window = this.$("twitternotifier-main-window");
    this._window.windowWidth = this._util.pref().getIntPref("windowWidth");
    this._window.windowHeight = this._util.pref().getIntPref("windowHeight");

    // Init session
    var password = this._util.getAccountInfo();
    if (password && this._util.pref().getBoolPref("login")) {
      this._util.notify("initSession", password);
    }
    else {
      this.logout();
    }

    this.changeTheme();
  },

  changeTheme: function() {
    var theme = this._util.pref().getCharPref("theme");
    var ios = this.Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService)
    var sss = this.Cc["@mozilla.org/content/style-sheet-service;1"].getService(Ci.nsIStyleSheetService)
    var uri = ios.newURI("chrome://twitternotifier/content/"+theme+".css", null, null);
    sss.loadAndRegisterSheet(uri, sss.AGENT_SHEET);
    //document.loadOverlay("chrome://twitternotifier/content/" + theme + ".xul", null);
    var style = this._window.input.style;
    if (theme == "black") {
      style.color = "#fff";
      style.backgroundColor = "#333";
      style.border = "1px solid #333";
    }
    else {
      style.color = "#000";
      style.backgroundColor = "#fff";
      style.border = "1px solid #fff";
    }
  },

  unload: function() {
    if (window.toolbar.visible == false) return;

    var observer = this.Cc["@mozilla.org/observer-service;1"].getService(this.Ci.nsIObserverService);
    observer.removeObserver(gTwitterNotifier, "twitternotifier-status");
  },

  focus: function(event) {
    if (!content) return;
    if (event.originalTarget == window || event.originalTarget == content.document) {
      this._hasFocus = true;
    }
    else if (event.originalTarget.tagName) {
      if (event.originalTarget.tagName == "INPUT" || event.originalTarget.tagName == "TEXTAREA") {
        this._focusInText = true;
      }
    }
  },

  blur: function(event) {
    if (!content) return;
    if (event.originalTarget == window || event.originalTarget == content.document) {
      this._hasFocus = false;
    }
    else if (event.originalTarget.tagName) {
      if (event.originalTarget.tagName == "INPUT" || event.originalTarget.tagName == "TEXTAREA") {
        this._focusInText = false;
      }
    }
  },

  observe: function(subject, topic, data) {

    if (topic != "twitternotifier-status") return;

    var msg = eval('(' + data + ')');
    if (this[msg.state]) {
      this[msg.state](msg.data);
    }
  },

  updateFriendsTimeline: function(data) {
    // Update balloon
    this._messageQueue = data.reverse();
    this.updateBalloon();
    this.updateTooltip();
    this.showMessage();
  },

  updateUsername: function(data) {
    this._username = data.screen_name.toLowerCase();
    this._window.currentUser = data.screen_name;
    this._window.currentUserIcon = data.profile_image_url;
    this.showMessage();
    if (this._window.isOpen) {
      this._util.notify("getRecent", {type:this._window.activeTab});
    }

    this.$("twitternotifier-menuitem-logout").setAttribute("disabled", false);
    this.$("twitternotifier-menuitem-update").setAttribute("disabled", false);
    this.$("twitternotifier-menuitem-account").setAttribute("label", this.getString("ChangeAccount"));

  },

  noUpdate: function(data) {
    this.updateTooltip();
    this.showMessage();
  },

  updateTooltip: function() {
    var elem = this.$("twitternotifier-last-update");
    var d = new Date();
    var h = d.getHours();
    if (h < 10) h = '0' + h;
    var m = d.getMinutes();
    if (m < 10) m = '0' + m;
    elem.value = "Last update: " + h + ":" + m;
  },

  accountChanged: function(data) {
    this._util.getAccountInfo();
    if (this._window.isOpen) {
      this._util.notify("getRecent", {type:this._window.activeTab});
    }
    this._util.accounts = this._util.getPassword();
  },

  authFail: function(data) {
    if (!this._prefWindow) {
      var prompt = this.Cc["@mozilla.org/embedcomp/prompt-service;1"].getService(this.Ci.nsIPromptService);
      var user = this._util.pref().getCharPref("currentUser");
      var msg = this.getFormattedString("AuthFail", [user]);
      prompt.alert(window, "TwitterFox", msg);
      this.logout();
    }
    else {
      this._prefWindow.focus();
    }
    this.showMessage(msg);
  },

  internalError: function(data) {
    this.showMessage(data);
    this.updateTooltip();
  },

  onTimeoutBalloon: function() {

    this.removeBalloon();
    if (this._messageQueue.length) {
      this.updateBalloon();
    }
    else {
      this.showUnreadCount(0);
    }
  },

  showMessage: function(message) {
    var elem = this.$("twitternotifier-status-tooltip");
    if (message) {
      this.setButtonState("error");
      elem.setAttribute("value", message);
    }
    else {
      this.setButtonState("active");
      elem.setAttribute("value", "TwitterFox");
    }
  },
  
  showUnreadCount: function(count) {
    var msg = {"state": "setUnreadCount", "data": count};

    var observer = this.Cc["@mozilla.org/observer-service;1"].getService(this.Ci.nsIObserverService);
    observer.notifyObservers(null, "twitternotifier-status", msg.toSource());
  },

  setUnreadCount: function(count) {
    var elem = this.$("twitternotifier-statusbar-text");
    var value;

    var notifier = this.Cc['@naan.net/twitternotifier;1'].getService(this.Ci.nsITwitterFox);
    var unread = notifier.getUnreadCount();
     
    if (count == 0) {
      value = unread || "";
    }
    else if (count > 0) {
      value = count + "/" + unread;
    }
    else {
      value = "";
    }
    elem.setAttribute("value", value);
  },

  setButtonState: function(state) {
    var btn = this.$("twitternotifier-statusbar-button");
    btn.setAttribute("state", state);
  },

  updateBalloon: function() {
    if (this._window.isOpen) {

      // Append message balloon to top of popup menu
      //
      var msgs = this._messageQueue;
      for (var i = 0; i < msgs.length; ++i) {
        var msg = msgs[i];
        var elem = this.createMessageBalloon(msg, true);
        elem.setAttribute("unread", "true");
        this._window.addBalloon(elem);
      }
      this.showUnreadCount(0);
      this._window.recalcScrollbar(false);
      return;
    }

    if (!this.canPopup()) {
      this.showUnreadCount(0);
      return;
    }

    //
    // Create new balloon
    //
    var unread = 0;
    for (var i = 0; i < this._messageQueue.length; ++i) {
      var msg = this._messageQueue[i];
      var user = msg.user ? msg.user : msg.sender;
      if (user && user.screen_name.toLowerCase() != this._username) {
        unread++;
      }
    }

    if (!this._util.pref().getBoolPref("popup")) {
      this.showUnreadCount(0);
      return;
    }

    if (this._util.pref().getBoolPref("dontPopupWhileTyping") && this._focusInText) {
      this.showUnreadCount(0);
      return;
    }

    this.showUnreadCount(unread);

    var count = this._messageQueue.length;

    if (count == 0) {
      return;
    }

    if (count > 5) {
      this._messageQueue = new Array();
      this.showNotice(this.getFormattedString("MoreThan5Tweets", [unread]));
    }
    else {
      this.showBalloon();
    }
  },

  showBalloon: function() {
    var elem = this.createMessageBalloon(this._messageQueue.shift(), false);
    elem.setAttribute("type", "balloon");
    this.popupBalloon(elem);
  },

  showNotice: function(msg) {
    var elem = document.createElement("vbox");
    elem.className = "twitternotifier-notice";
    elem.setAttribute("value", msg);

    this.popupBalloon(elem);
  },

  popupBalloon: function(elem) {

    if (!this.canPopup()) {
      return;
    }

    // Fix some style for balloon popup
    elem.style.margin = "0px";
    elem.style.MozBorderRadius = "0px";
    elem.setAttribute("unread", "true");

    var interval = this._util.pref().getIntPref("popup-interval");
    if (!interval) {
      interval = 3;
    }
    var panel = this.$("twitternotifier-popup");
    panel.appendChild(elem);
    panel.showBalloon(interval);
  },

  openWindow: function(data) {

    if (!this.isActiveWindow()) {
      return;
    }
    // remove balloon
    this.removeBalloon();

    this.showUnreadCount(0);

    var msgs = data.msgs;
    var type = data.type;

    this._window.removeStatuses();

    for (var i in msgs) {
      if (msgs.hasOwnProperty(i)) {
        var elem = this.createMessageBalloon(msgs[i], true);
        this._window.appendChild(elem);
      }
    }

    this._window = this.$("twitternotifier-main-window");
    this._window.windowWidth = this._util.pref().getIntPref("windowWidth");
    this._window.windowHeight = this._util.pref().getIntPref("windowHeight");

    this._window.show();

    if (navigator.platform.match("Mac")) {
      this._window.input.style.padding = "0px";
    }

    this._window.setActiveTab(data);
    //
    // workaround to make sure re-calc scrollbar size
    //
    var obj = this;
    setTimeout(function() {obj._window.recalcScrollbar(true);}, 100);
  },

  closePopup: function(force) {

    if (this._window.isOpen) {
      this._util.notify("markRead", {type: this._window.activeTab});
      this.showUnreadCount(0);
      if (force) {
        this._window.hide();
      }
    }
  },

  removeBalloon: function() {
    try {
      var panel = this.$("twitternotifier-popup");
      panel.hideBalloon();
    }
    catch (e) {}
  },

  createMessageBalloon: function(msg, highlight) {

    var elem = document.createElement("vbox");
    elem.className = "twitternotifier-status";
    elem.id = "tooltip-balloon-" + msg.id;

    elem.setAttribute("messageId", msg.id);
    elem.setAttribute("favorited", msg.favorited);

    var fontSize = this._util.pref().getIntPref("fontSize");
    elem.style.fontSize = fontSize + "px";

    var user = msg.user ? msg.user : msg.sender;

    elem.setAttribute("attr", msg.type);
    elem.setAttribute("type", msg.type);

    elem.setAttribute("profile_image_url", user.profile_image_url);

    try {
      elem.setAttribute("href", TWITTER_TOP_URL + user.screen_name);

      elem.setAttribute("screen_name", user.screen_name);
      elem.setAttribute("name", user.name);
      if (highlight) {
        elem.setAttribute("unread", msg.unread);
      }

      var textnode = this.replaceLinkText(msg.text);
      elem.setAttribute("text", msg.text);
      if (textnode.getAttribute("attr") == "replies") {
        elem.setAttribute("attr", "replies");
      }

      var info = document.createElement("description");
      info.className = "twitternotifier-status-info";
      info.style.fontSize = (fontSize - 1) + "px";

      var time = this.createAnchorText("http://twitter.com/" + user.screen_name + "/statuses/" + msg.id, 
                                       this.getLocalTimeForDate(msg.created_at),
                                       "twitternotifier-permalink");
      info.appendChild(time);
      info.style.display = "block";
      if (msg.source) {
        if (msg.source.match(/<a href\=\"(.*)\">(.*)<\/a>/)) {
          info.appendChild(document.createTextNode(" from "));
          var source = this.createAnchorText(RegExp.$1, RegExp.$2, "twitternotifier-source-link");
          info.appendChild(source);
        }
      }
      if (msg.in_reply_to_status_id) {
        info.appendChild(document.createTextNode(" "));
        var reply = this.createAnchorText("http://twitter.com/" + msg.in_reply_to_screen_name + "/statuses/" + msg.in_reply_to_status_id,
                                          "in reply to " + msg.in_reply_to_screen_name,
                                          "twitternotifier-source-link");
        info.appendChild(reply);
      }

      elem.appendChild(textnode);
      elem.appendChild(info);

      // set user profile
      var profile = document.createElement("profile");
      profile.setAttribute("name", user.name);
      var pmsg = 
        ((user.location) ? user.location + "\n" : "") + 
        ((user.url) ? user.url + "\n" : "") +
        user.followers_count + " followers\n" +
        user.friends_count + " following\n" +
        user.statuses_count + " updates";
      profile.appendChild(document.createTextNode(pmsg));

      elem.appendChild(profile);

    }
    catch (e) {
      this.log("Failed to create message balloon: " + e.message);
    }

    return elem;
  },

  setFocusToInput: function() {
    this._window.input.value = this._window.text;
    this._window.input.select();
    var pos = this._window.input.value.length;
    try {
      this._window.input.setSelectionRange(pos, pos);
    }
    catch (e) {};
  },

  hideProgress: function() {
    this._window.showProgress(false);

    var input = this._window.input;
    input.select();
    input.focus();
  },

  onSendMessage: function() {

    var input = this._window.input;

    // Ignore autocomplete result
    var re = new RegExp('^@[^ ]+$');
    if (re.test(input.value)) {
  	  return true;
    }

    if (input.value == '') {
      return false;
    }

    // convert URLs to tinyURL if the URLs are longer than 30 characters.
    var result = [];
    var tinyURL = [];
    var text = input.value;

    var pat = TWITTERFOX_URL_REGEXP;
    var re = /[!.,;:)}\]]+$/;

    if (text.length > 140) {
      while (pat.exec(text) != null) {
        result.push(RegExp.leftContext);
        var url = RegExp.$1;
        text = RegExp.rightContext;
        if (re.test(url)) {
          text = RegExp.lastMatch + text;
          url = url.replace(re, '');
        }
        result.push(url);

        pat.lastIndex = 0;

        if (url.length > 25 &&
            !url.match(/tinyurl\.com/i)) {
          tinyURL.push(url);
        }
      }
    }
    if (text) {
      result.push(text);
    }

    this._sendText = result.join('').replace(/[\n\r]/, '');

    if (tinyURL.length) {
      this._window.showProgress(true);

      this._sendMsgs = result;
      this._tinyURLs = tinyURL;
      this.makeTinyURL();
      return false;
    }

    if (this._sendText.length > 140) {
      return false;
    }

    this._window.showProgress(true);
    this._util.notify("sendMessage", {status: this._sendText, inReplyTo:this._inReplyTo});
    return true;
  },

  makeTinyURL: function() {

    var url = this._tinyURLs[0];

    var req = new XMLHttpRequest;

    req.open('GET', TINYURL_CREATE_API + encodeURIComponent(url), true);
    var target = this;
    req.onload  = function() {target.onloadMakeTinyURL(req, url)};
    req.onerror = function() {target.onerrorMakeTinyURL()};
    req.send(null);

  },

  onloadMakeTinyURL: function(req, url) {
    if (req.status == 200) {
      this._tinyURLs.shift();

      for (var i = 0; i < this._sendMsgs.length; ++i) {
        if (this._sendMsgs[i] == url) {
          this._sendMsgs[i] = req.responseText;
          break;
        }
      }
      this._sendText = this._sendMsgs.join('').replace(/[\n\r]/, '');
    }

    if (this._tinyURLs.length) {
      this.makeTinyURL();
    }
    else {
      if (this._sendText.length > 140) {
        this.retryInput();
      }
      else {
        this._util.notify("sendMessage", {status: this._sendText, inReplyTo:this._inReplyTo});
      }
    }
  },

  onerrorMakeTinyURL: function() {
    this.retryInput();
    this.showMessage(this.getString("SendMessageError"));
  },

  updateTinyURL: function(param) {
    var link = document.getElementById(param.messageId + "-" + param.originalURL);
    link.setAttribute("href", param.url);
    var elem = document.getElementById(param.messageId);
    elem.setAttribute("_tooltiptext", param.url);
  },

  sentTweet: function(msg) {
    this.sentMessage();
    this._messageQueue = [msg];
    this.showMessage();

    // Workaround #110 for firefox 2:
    //   Add delay to open ballon popup to avoid the popup split each word into a line
    //
    setTimeout("gTwitterNotifier.updateBalloon()", 300);
  },

  sentMessage: function() {
    this._window.resetText();
    this.hideProgress();
    this.closePopup(false);
    this._inReplyTo = 0;
  },

  errorOnSendMessage: function(msg) {
    this._window.setAttribute("errorMessage", this.getString("SendMessageError"));
    this._window.showErrorMessage(true);
    setTimeout("gTwitterNotifier.retryInput()", 2 * 1000);
    this.showMessage(this.getString("SendMessageError"));
  },

  retryInput: function() {
    this._window.message = this._sendText;
    this._window.showErrorMessage(false);

    var input = this._window.input;
    input.select();
    input.focus();
  },

  onClickStatusbarIcon: function(e) {
    if (e.button == 0) {
      this.onOpenPopup();
    }
  },

  onOpenPopup: function() {
    if (this._util.pref().getBoolPref("login") == false) {
      this.onPreference(null);
      return;
    }

    this.removeBalloon();
    this._messageQueue = new Array();
    this.showUnreadCount(0);


    if (this._window.isOpen) {
      this.closePopup(true);
    }
    else {
      this._util.notify("getRecent", {type:"timeline"});
    }
  },

  changeTab: function(name) {
    this._util.notify("markRead", {type: this._window.activeTab});
    this._util.notify("getRecent", {type: name});
  },

  onRevertText: function(text) {
    if (text.value == "") {
      this.closePopup(true);
    }
    else {
      this._window.resetText();
      text.select();
    }
    this._inReplyTo = 0;
    return true;
  },

  onInsertURL: function() {
    var clipboard = this.Cc["@mozilla.org/widget/clipboardhelper;1"].getService(this.Ci.nsIClipboardHelper);

    clipboard.copyString(content.document.location.toString());

    var text = this.$("twitternotifier-message-input");
    if (!text) {
      return;
    }

    text.focus();
    goDoCommand("cmd_paste");
  },

  onBalloonClick: function(e) {
    var node = e.target;
    if (e.button != 2) {
      var url = node.getAttribute('href');

      if (url) {
        this.showMessage();
        this.openURL(url);
        this.closePopup(false);
      }
    }
    else {
      var menu = this.$("twitternotifier-status-menupopup");
      while (!node.id) {
        node = node.parentNode;
      }
      menu.node = node;

      menu.childNodes[1].disabled = false;
      menu.childNodes[2].disabled = false;
      menu.lastChild.disabled = true;

      if (this._window.activeTab == 'messages') {
        menu.lastChild.disabled = false;
        menu.childNodes[1].disabled = true;
        menu.childNodes[2].disabled = true;
      }
      if (this._username.toLowerCase() == node.getAttribute("screen_name").toLowerCase()) {
        menu.lastChild.disabled = false;
      }
      // openPopupAtScreen is available on only Fx3, not Fx2, so avoid JS error.
      //
      try {
        menu.openPopupAtScreen(e.screenX, e.screenY, true);
      }
      catch (e) {}
    }
  },

  copyTweet: function(target) {

    var clipboard = this.Cc["@mozilla.org/widget/clipboardhelper;1"].getService(this.Ci.nsIClipboardHelper);

    var node = target.parentNode.node;
    while (node) {
      var id = node.id;
      if (node.id && node.id.match(/tooltip-balloon/)) break;
      node = node.parentNode;
    }
    if (node == null) return;
    var tweet = node;
    clipboard.copyString(tweet.getAttribute("text"));
  },

  retweet: function(target) {
    var node = target.parentNode.node;
    while (node) {
      var id = node.id;
      if (node.id && node.id.match(/tooltip-balloon/)) break;
      node = node.parentNode;
    }
    if (node == null) return;
    var tweet = node;
    
    var text = "RT: @" + tweet.getAttribute("screen_name") + ": " + tweet.getAttribute("text");

    this._window.text = text;
    this._window.input.value = text;
    this._window.showTextBox(true);

    this._window.setAttribute("charcount", text.length == 0 ? "" : 140 - text.length);
    this._window.setAttribute("charover", (text.length > 140) ? 1 : 0);

    this._window.input.focus();
    this._window.input.value = text;
  },

  deleteTweet: function(target) {
    var tweet = target.parentNode.node;
    this._util.notify("deleteTweet", 
                      {id:tweet.getAttribute("messageId"), type:this._window.activeTab});
  },

  messageDeleted: function(obj) {
    for (var i = 0; i < this._window.childNodes.length; ++i) {
      var e = this._window.childNodes[i];
      if (e.getAttribute("messageId") == obj.id) {
        this._window.removeChild(e);
        break;
      }
    }
  },

  resetInReplyTo: function() {
    this._inReplyTo = 0;
  },

  onReply: function(msg) {
    if (!this._window.isOpen) {
      this.onOpenPopup();
    }
    
    this._inReplyTo = msg.getAttribute("messageId");

    var input = this._window.input;
    var reply = (msg.getAttribute("attr") == "messages" ? "d " : "@") + msg.getAttribute("name") + " ";

    var text = reply + input.value;
    this._window.text = text;
    input.value = text;
    this._window.showTextBox(true);

    this._window.setAttribute("charcount", text.length == 0 ? "" : 140 - text.length);
    this._window.setAttribute("charover", (text.length > 140) ? 1 : 0);

    input.focus();
    input.value = text;
  },

  onFavorite: function(msg) {
    var messageId = msg.getAttribute("messageId");
    var method;
    if (msg.getAttribute("favorited") != "true") {
      method = "create";
    }
    else {
      method = "destroy";
    }
    
    this._util.notify("setFavorite",{id: messageId, method: method});
  },

  updateFavorite: function(msg) {
    var elem = this.$("tooltip-balloon-" + msg.id);
    if (elem) {
      elem.setAttribute("favorited", msg.state);
    }
  },

  openURL: function(url) {
    var tabbrowser = gBrowser;
    var tabs = tabbrowser.tabContainer.childNodes;
    for (var i = 0; i < tabs.length; ++i) {
      var tab = tabs[i];
      try {
        var browser = tabbrowser.getBrowserForTab(tab);
        if (browser) {
          var doc = browser.contentDocument;
          var loc = doc.location.toString();
          if (loc == url) {
            gBrowser.selectedTab = tab;
            return;
          }
        }
      }
      catch (e) {
      }
    }
    
    // There is no tab. open new tab...
    var tab = gBrowser.loadOneTab(url);
  },

  updateStatuses: function(e) {
    this._util.notify("updateTweets");
  },

  markAllRead: function(e) {
    this.showUnreadCount(-1);
    
    for (var i = 0; i < this._window.childNodes.length; ++i) {
      var e = this._window.childNodes[i];
      e.setAttribute("unread", false);
    }
    this._util.notify("markAllRead");
    if (this._window.isOpen) {
      this._window.markRead();
    }
  },

  onPreference: function(e) {
    if (this._prefWindow) {
      this._prefWindow.focus();
      return true;
    }

    this._prefWindow = window.openDialog("chrome://twitternotifier/content/preference.xul", 
                                         "_blank",
                                         "chrome,resizable=no,dependent=yes");
    return true;
  },

  onTogglePopup: function(e) {
    var menu = this.$("twitternotifier-menuitem-togglepopup");
    var flag = this._util.pref().getBoolPref("popup");
    menu.setAttribute("checked", !flag);
    this._util.pref().setBoolPref("popup", !flag);
  },

  onLogout: function() {
    this._util.notify("logout");
  },

  logout: function() {
    this.showUnreadCount(-1);
    this.setButtonState("");
    this.$("twitternotifier-menuitem-logout").setAttribute("disabled", true);
    this.$("twitternotifier-menuitem-update").setAttribute("disabled", true);
    this.$("twitternotifier-menuitem-account").setAttribute("label", this.getString("SignIn"));

    this.checkMenuItem(null, "twitternotifier-accounts");
    this.checkMenuItem(null, "twitternotifier-accounts-popup");

    // Close balloon and popup window, reset window settings
    this.removeBalloon();
    this.closePopup(true);
    this._util.pref().setBoolPref("login", false);
  },

  onAccountMenuShowing: function(menu) {

    this._util.accounts = this._util.getPassword();
    var currentUser = this._util.pref().getCharPref("currentUser");
    this.removeAllChild(menu);

    var loggedIn = this._util.pref().getBoolPref("login");

    for (var user in this._util.accounts) {
      if (this._util.accounts.hasOwnProperty(user)) {
        var item = document.createElement("menuitem");
        item.setAttribute("label", user);
        item.setAttribute("type", "radio");
        item.setAttribute("oncommand", "gTwitterNotifier.onChangeAccount(this.label)");

        if (currentUser == null) {
          currentUser = user;
        }

        if (user == currentUser && loggedIn) {
          item.setAttribute("checked", true);
        }
        menu.appendChild(item);
      }
    }

  },

  onChangeAccount: function(user) {

    var currentUser = this._util.pref().getCharPref("currentUser");

    if (user != currentUser || this._util.pref().getBoolPref("login") == false) {

      // Close balloon and popup window, reset window settings
      this.removeBalloon();
      this.showUnreadCount(-1);
      this._window.removeStatuses();

      this._util.pref().setBoolPref("login", true);

      this.$("twitternotifier-menuitem-logout").setAttribute("disabled", false);
      this.$("twitternotifier-menuitem-update").setAttribute("disabled", false);
      this.$("twitternotifier-menuitem-account").setAttribute("label", this.getString("ChangeAccount"));

      this._util.accounts = this._util.getPassword();

      this._util.pref().setCharPref("currentUser", user);
      this._util.notify("changeAccount", {user: user, pass: this._util.accounts[user]});
    }
  },

  //
  // Private utilities
  //
  canPopup: function() {
    if (!this._hasFocus) return false;
    return this.isActiveWindow();
  },

  isActiveWindow: function() {
    var wm = this.Cc["@mozilla.org/appshell/window-mediator;1"].getService(this.Ci.nsIWindowMediator);
    var win = wm.getMostRecentWindow("");
    return (win == window) ? true : false;
  },

  replaceLinkText : function(text) {

    var escape = this.Cc["@mozilla.org/feed-unescapehtml;1"].getService(this.Ci.nsIScriptableUnescapeHTML);
    text = escape.unescape(text.replace(/&amp;/g,"&"));

    var elem = document.createElement("description");
    elem.className = "twitternotifier-message-body";

    var pat = TWITTERFOX_URL_REGEXP;
    var re = /[!.,;:)}\]]+$/;

    while (pat.exec(text) != null) {
      var left = RegExp.leftContext;
      var url = RegExp.$1;
      text = RegExp.rightContext;
      if (re.test(url)) {
        text = RegExp.lastMatch + text;
        url = url.replace(re, '');
      }

      this.convertFollowLink(elem, left);

      var urltext = url;
      if (url.length > 27) {
        urltext = url.substr(0, 27) + "...";
      }
      var anchor = this.createAnchorText(url, urltext);
      elem.appendChild(anchor);
      pat.lastIndex = 0;
    }

    if (text) {
      this.convertFollowLink(elem, text);
    }

    return elem;
  },

  convertFollowLink: function(elem, text) {
    var pat = /@(\w+)/;

    while(pat.exec(text) != null) {
      var username = RegExp.$1;
      var atUsername = RegExp.lastMatch;
      text = RegExp.rightContext;

      elem.appendChild(document.createTextNode(RegExp.leftContext));

      var a = this.createAnchorText("http://twitter.com/" + username, atUsername);
      elem.appendChild(a);
      pat.lastIndex = 0;
      if (username.toLowerCase() == this._username) {
        elem.setAttribute("attr", "replies");
      }
    }
    if (text) {
      elem.appendChild(document.createTextNode(text));
    }
  },

  createAnchorText: function(link, text, classname) {
      var anchor = document.createElement("a");
      if (classname) {
        anchor.className = classname;
      }
      else {
        anchor.className = "twitternotifier-hyperlink";
      }
      anchor.setAttribute("href", link);

      anchor.appendChild(document.createTextNode(text));

      return anchor;
  },

  checkMenuItem: function(user, container) {
    var menu = this.$(container);
    if (menu) {
      for (var i = 0; i < menu.childNodes.length; ++i) {
        if (menu.childNodes[i].getAttribute("label") == user) {
          menu.childNodes[i].setAttribute("checked", true);
        }
        else {
          menu.childNodes[i].setAttribute("checked", false);
        }
      }
    }
  },

  initKeyConfig: function() {
    // setup short cut keys
    var key = ["togglePopup", "insertURL"];

    for (var i = 0; i < key.length; ++i) {
      var pref = this._util.pref().getCharPref(key[i]);
      var params = pref.split(/,/);

      var elem = this.$("twitternotifier-key-" + key[i]);

      if (elem) {
        if (params[0])
          elem.setAttribute("key", params[0]);
        if (params[1])
          elem.setAttribute("keycode", params[1]);
        elem.setAttribute("modifiers", params[2]);
      }
    }

  },

  removeAllChild: function(obj) {
    while(obj.firstChild) obj.removeChild(obj.firstChild);
  },

  getLocalTimeForDate: function(time) {

    var system_date = new Date(time);
    var user_date = new Date();

    var delta_minutes = Math.floor((user_date - system_date) / (60 * 1000));
    if (Math.abs(delta_minutes) <= (7 * 24 * 60)) {
      var distance = this.distanceOfTimeInWords(delta_minutes);
      if (delta_minutes < 0) {
        return this.getFormattedString("DateTimeFromNow", [distance]);
      } else {
        return this.getFormattedString("DateTimeAgo", [distance]);
      }
    } else {
      return this.getFormattedString("OnDateTime", [system_date.toLocaleDateString()]);
    }
  },

  distanceOfTimeInWords: function(minutes) {
    if (minutes.isNaN) return "";

    var index;

    minutes = Math.abs(minutes);
    if (minutes < 1)         index = 'LessThanAMinute';
    else if (minutes < 50)   index = (minutes == 1 ? 'Minute' : 'Minutes');
    else if (minutes < 90)   index = 'AboutOneHour';
    else if (minutes < 1080) {
      minutes = Math.round(minutes / 60);
      index = 'Hours';
    }
    else if (minutes < 1440) index = 'OneDay';
    else if (minutes < 2880) index = 'AboutOneDay';
    else {
      minutes = Math.round(minutes / 1440);
      index = 'Days';
    }
    return this.getFormattedString(index, [minutes]);
  },

  log: function(msg) {
    var pref = this.Cc['@mozilla.org/preferences-service;1'].getService(this.Ci.nsIPrefBranch);

    if (pref.getBoolPref("extensions.twitternotifier.debug")) {
      if (this._console == null) 
        this._console = this.Cc["@mozilla.org/consoleservice;1"].getService(this.Ci.nsIConsoleService);
      this._console.logStringMessage(msg);
      dump(msg + "\n");
    }
  },

  getString: function(key) {
    return this.$("twitternotifier-strings").getString(key);
  },

  getFormattedString: function(key, params) {
    return this.$("twitternotifier-strings").getFormattedString(key, params);
  },
   
  dumpall: function(obj) {
    for (var i in obj) {
      if (typeof obj[i] == 'object') {
        dump(i + ":\n");
        this.dumpall(obj[i]);
      }
      else if (typeof obj[i] != 'function') {
        dump(i + ":" + obj[i] + "\n");
      } 
    }
  },

  Cc: this.Components.classes,
  Ci: this.Components.interfaces,

  $: function(name) {
    return document.getElementById(name);
  }

};

//
// Create instance and add event listener.
//
var gTwitterNotifier = new TwitterNotifier();

window.addEventListener("load",   function(e) { gTwitterNotifier.load(e);   }, false);
window.addEventListener("unload", function(e) { gTwitterNotifier.unload(e); }, false);
window.addEventListener("focus",  function(e) { gTwitterNotifier.focus(e);  }, true);
window.addEventListener("blur",   function(e) { gTwitterNotifier.blur(e);   }, true);
